package com.incubator.game;

import com.incubator.common.env.Environment;
import com.incubator.common.filter.SensitiveDictionary;
import com.incubator.common.game.AbstractService;
import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.common.net.NetServer;
import com.incubator.common.net.netty.handler.websocket.binary.WebSocketServerBinaryAdapterHandler;
import com.incubator.common.net.netty.server.NettyServer;
import com.incubator.common.thread.NamedThreadFactory;
import com.incubator.common.util.PropertiesUtil;
import com.incubator.core.net.http.HttpRequestHandler;
import com.incubator.game.listener.HttpInnerListener;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.handler.InitalizerHttpServerHandler;
import com.incubator.core.net.handler.InitalizerProtocolHandler;
import com.incubator.core.quartz.QuartzServer;
import com.incubator.game.data.entity.GlobalInfoPO;
import com.incubator.game.data.jdbc.LSDBHandler;
import com.incubator.game.listener.PublicListener;
import com.incubator.game.player.GPlayer;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.GameDataUtil;
import io.netty.util.CharsetUtil;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledThreadPoolExecutor;

public class GGame extends AbstractService {

    private static final Logger logger;

    public static ScheduledThreadPoolExecutor scheduleExec = new ScheduledThreadPoolExecutor(2, new NamedThreadFactory("ServerExecutor", true));
    /** 运行环境 **/
    public static final Environment env = Environment.getInstance();

    private static final LSDBHandler lsDBHandler = new LSDBHandler();

    /** 服务器设置目录 **/
    public static PropertiesUtil SERVER_SETTINGS;
    public static PropertiesUtil DB_SETTINGS;
    public static String CONFIG_FILE = "config.properties";
    public static String DB_FILE = "db.properties";

    public static String DB_FILE_1 = "conf.ds.xml";
    public static String EVENT_FILE = "event.xml";

    /** 服务器id **/
    public static int SERVER_ID = 1001;
    /** 服务器绑定IP **/
    public static String BIND_HOST;
    /** 服务器绑定端口 **/
    public static int BIND_PORT;
    /** 绑定的http ip **/
    public static String BIND_HTTP_HOST;
    /** 绑定的http 端口 **/
    public static int BIND_HTTP_PORT;
    /** 中控服绑定ip **/
    public static String CENTER_HOST;
    /** 中控服绑定端口 **/
    public static int CENTER_PORT;
    /** 策划JSON数据目录 **/
    public static String DIR_GAME_DATA = "";

    /** 玩家心跳开关 **/
    public static boolean heartEnable = false;

    /** 游戏世界的时间
     *  此两个时间最小精度为秒,不能用于超过精度一秒的时间计算
     *  (APP_TIME每隔一秒执行一下,所以最小精度一秒),方法为JobFactory.addScheduleJob
     * */
    public static volatile long APP_TIME = System.currentTimeMillis();

    public static volatile int APP_TIME_SEC = (int) (APP_TIME / 1000);

    /** 是否注册中控 **/
    public static boolean isRegCenter;

    /** 存储连接 **/
    public static Set<Connection> publicClients = new HashSet<>();

    /** ws消息事件 **/
    public static Map<Integer, NetHandler> handlers = new HashMap<>();

    /** http消息事件 **/
    public static Map<String, HttpRequestHandler> httpHandlers = new HashMap<>();

    /** 本服所有在线玩家 */
    public static Map<String, GPlayer> onlinePlayers = new ConcurrentHashMap<>();

    /** http玩家缓存 **/
//    public static Map<String, GPlayer> httpPlayerCatch = new ConcurrentHashMap<>();

    public static GlobalInfoPO globalInfoPO = new GlobalInfoPO();

    private static volatile GGame instance;

    static {
        // 日志加载
        Log4jUtil.initLogConfig(env.getLogConfigFile());
        logger = Log4jUtil.getLogger(GGame.class);
    }

    public static GGame getInstance() {
        if (instance == null) {
            synchronized (GGame.class) {
                if (instance == null) {
                    instance = new GGame();
                }
            }
        }
        return instance;
    }

    public static void addPlayer(GPlayer player) {
        onlinePlayers.put(player.getId(), player);
    }

    public static LSDBHandler getDBHandler() {
        return lsDBHandler;
    }

    @Override
    protected void doStart() throws Throwable {
        // 1.初始化Properties配置
        this.initProperties();
        // 2.初始化数据库连接池
//        DBFactory.init();
        // 3.初始化游戏数据
        this.initGameData();
        // 4.初始化定时任务
        this.initQuartzTask();
        // 5.初始化消息事件
        this.initHandler();
        // 6.初始化网络服务
        this.initNetService();
    }

    @Override
    protected void doClose() throws Throwable {
        logger.info("服务器维护中, 已关闭服务器!");
    }

    /**
     * 1.初始化Properties配置
     *
     * @throws Exception
     */
    public void initProperties() throws Exception {
        logger.info("初始化Properties配置...");
        SERVER_SETTINGS = new PropertiesUtil(env.getConfDir() + "/" + CONFIG_FILE);
        DB_SETTINGS = new PropertiesUtil(env.getConfDir() + "/" + DB_FILE);

        SERVER_ID = Integer.parseInt(SERVER_SETTINGS.getProperty("SERVER_ID", "1001"));

        BIND_HOST = SERVER_SETTINGS.getProperty("BIND_HOST", "0.0.0.0");
        BIND_PORT = Integer.parseInt(SERVER_SETTINGS.getProperty("BIND_PORT", "9000"));

        BIND_HTTP_HOST = SERVER_SETTINGS.getProperty("BIND_HTTP_HOST", "0.0.0.0");
        BIND_HTTP_PORT = Integer.parseInt(SERVER_SETTINGS.getProperty("BIND_HTTP_PORT", "9001"));

        CENTER_HOST = SERVER_SETTINGS.getProperty("CENTER_HOST", "0.0.0.0");
        CENTER_PORT = Integer.parseInt(SERVER_SETTINGS.getProperty("CENTER_PORT", "4000"));

        DIR_GAME_DATA = SERVER_SETTINGS.getProperty("DIR_GAME_DATA", "");
    }

    /**
     * 3.初始化游戏数据
     */
    public void initGameData() {
        //加载策划配置表
        GameDataUtil.initData();
        //加载数据库数据
//        this.initAllBlobGameData();
        //初始化 房间服务
        RoomService.getInstance().init();
    }

//    private ProtoData.GlobalAllBlobData.Builder initAllBlobGameData() {
//        ProtoData.GlobalAllBlobData.Builder allBlobData = ProtoData.GlobalAllBlobData.newBuilder();
//        try {
//            globalInfoPO = getDBHandler().getGlobalInfoByServer(SERVER_ID);
//            if (globalInfoPO == null) {
//                globalInfoPO = new GlobalInfoPO();
//                globalInfoPO.server_id = SERVER_ID;
//            }
//            if (globalInfoPO.allBlobData == null) {
//                globalInfoPO.allBlobData = ProtoData.GlobalAllBlobData.newBuilder().build().toByteArray();
//                getDBHandler().insertGlobal(globalInfoPO);
//            }
//            allBlobData = ProtoData.GlobalAllBlobData.parseFrom(globalInfoPO.allBlobData).toBuilder();
//        } catch (InvalidProtocolBufferException e) {
//            logger.error("初始化服务器数据出错...{}", e.getMessage());
//        }
//        return allBlobData;
//    }

    /**
     * 4.todo 初始化定时器任务
     **/
    private void initQuartzTask() {
        logger.info("初始化定时器任务...");
        try {
//            QuartzServer task = new QuartzServer();
//            task.addJob(new FlushDB());
//            task.addJob(new RefreshNewDay());
        } catch (Exception e) {
            logger.error("初始化定时器任务出错...{}", e.getMessage());
        }
    }

    /**
     * 5.初始化消息事件
     */
    private void initHandler() {
        logger.info("初始化消息事件...");
        File f = new File(env.getConfDir() + "/" + EVENT_FILE);
        if (f.exists() && f.isFile()) {
            try {
                FileInputStream inputStream = new FileInputStream(f);
                SAXReader saxReader = new SAXReader();
                Document document = saxReader.read(inputStream);
                Element root = document.getRootElement();
                List<?> elements = root.elements("onDate");
                String _key = null;
                for (Object element : elements) {
                    Element e = (Element) element;
                    String clazz = e.attributeValue("handleClass");

                    _key = e.attributeValue("id");
                    if (StringUtils.isNotEmpty(_key)) {
                        int id = Integer.parseInt(_key);
                        handlers.put(id, (NetHandler) Class.forName(clazz).newInstance());
                    }

                    _key = e.attributeValue("http");
                    if (StringUtils.isNotEmpty(_key)) {
                        httpHandlers.put(_key, (HttpRequestHandler) Class.forName(clazz).newInstance());
                    }
                }
            } catch (Exception e) {
                logger.error("初始化消息事件出错 : {}", e.getMessage());
            }
        }
    }

    /**
     * 6.初始化网络服务
     **/
    private void initNetService() {
        logger.info("初始化网络服务...");
        // websocket端口
        NetServer publicServer = new NettyServer(GGame.BIND_HOST, GGame.BIND_PORT,
                new WebSocketServerBinaryAdapterHandler("/ws", new InitalizerProtocolHandler(new PublicListener())));
        publicServer.start();

        // http端口
        NetServer innerHttpServer = new NettyServer(GGame.BIND_HTTP_HOST, GGame.BIND_HTTP_PORT,
                new InitalizerHttpServerHandler(new HttpInnerListener()));
        innerHttpServer.start();

        // 开启连接中控服务端
//        NetClient centerClient = new NettyClient(GGame.CENTER_HOST, GGame.CENTER_PORT,
//                new WebSocketServerBinaryAdapterHandler("/ws", new InitalizerProtocolHandler(new CenterClientListener())));
//        centerClient.enableReconnect(scheduleExec, TimeUnit.SECONDS.toMillis(5));
//        centerClient.setName("centerClient");
//        centerClient.start();
    }
}
